<!DOCTYPE html>
<html lang="en">
<head>
	<meta charset="UTF-8">
	<title>Title</title>
</head>
<body>
<div>API took {{ dt }}</div>
<div>API user details:</div>
<ul>
	{% for k,v in api_user.items %}
		{% if k == "__metadata" %}
		{% elif k == "ThumbnailPicture" %}
			<img src="data:image/jpg;base64, {{ v }}"/>
		{% else %}
			<li>{{ k }}: {{ v }}</li>
		{% endif %}
	{% endfor %}
</ul>

<div>Division:</div>
<ul>
	{% for k,v in division.items %}
		{% if k == "__metadata" %}
		{% else %}
			<li>{{ k }}: {{ v }}</li>
		{% endif %}
	{% endfor %}
</ul>

<div>Webhooks:</div>
{% for webhook in webhooks %}
	<ul>
		{% for k,v in webhook.items %}
			<li>{{ k }}: {{ v }}</li>
		{% endfor %}
	</ul>
{% empty %}
	<span>No Webhooks configured.</span>
{% endfor %}

</body>
</html>